import React, { useEffect, useRef, useState } from 'react';
import Plot from 'react-plotly.js';
import { DropdownSelector } from '@/lib/components/DropdownSelector/DropdownSelector';
import { Vis1DInputData } from './types';
import { preparePlotDataAndLayoutBar } from '../prepareData/preparePlotBar';
import { Input } from '@/lib/components/inputs';

type Vis1DBarProps = {
  xData: Vis1DInputData;
  yData: Vis1DInputData;
  zData: Vis1DInputData;
  groupDataByTime?: string;
  showAxis: boolean;
};

export const Vis1DBar: React.FC<Vis1DBarProps> = ({ xData, yData, zData, showAxis, groupDataByTime }) => {
  const internalRef = useRef<HTMLDivElement>(null);
  const [divSize, setDivSize] = useState({ width: 0, height: 0 });

  useEffect(() => {
    const handleResize = () => {
      if (internalRef.current) {
        const { width, height } = internalRef.current.getBoundingClientRect();
        setDivSize({ width, height });
      }
    };

    handleResize();
    window.addEventListener('resize', handleResize);
    if (internalRef.current) {
      new ResizeObserver(handleResize).observe(internalRef.current);
    }

    return () => {
      window.removeEventListener('resize', handleResize);
    };
  }, []);

  const { data, layout } = preparePlotDataAndLayoutBar(xData, yData, zData, showAxis, groupDataByTime);

  return (
    <div className="h-full w-full flex items-center justify-center overflow-hidden relative" ref={internalRef}>
      <Plot
        data={data}
        config={{
          responsive: true,
          scrollZoom: false,
          displayModeBar: false,
          displaylogo: false,
        }}
        layout={{
          ...layout,
          width: divSize.width,
          height: divSize.height,
          dragmode: false,
        }}
      />
    </div>
  );
};

interface Vis1DSettingsBarProps {
  settings: any;
  updateSettings: (updatedSettings: any) => void;
  attributeOptions: { name: string; type: string }[];
  filterYLabel: string;
  setFilterYLabel: React.Dispatch<React.SetStateAction<string>>;
  filterZLabel: string;
  setFilterZLabel: React.Dispatch<React.SetStateAction<string>>;
}

export const Vis1DSettingsBar = ({
  settings,
  updateSettings,
  attributeOptions,
  filterYLabel,
  setFilterYLabel,
  filterZLabel,
  setFilterZLabel,
}: Vis1DSettingsBarProps) => {
  return (
    <div>
      <div className="mb-2">
        <DropdownSelector
          label="Y Axis"
          selectedValue={settings.yAxisLabel.name}
          options={attributeOptions}
          filter={filterYLabel}
          setFilter={setFilterYLabel}
          onChange={selected => {
            updateSettings({ yAxisLabel: { name: selected.name, type: selected.type } });
          }}
        />
        <DropdownSelector
          label="Z Axis"
          selectedValue={settings.zAxisLabel.name}
          options={attributeOptions}
          filter={filterZLabel}
          setFilter={setFilterZLabel}
          onChange={selected => {
            updateSettings({ zAxisLabel: { name: selected.name, type: selected.type } });
          }}
        />
        {(settings.xAxisLabel.name?.includes('Datum') ||
          settings.xAxisLabel.type === 'date' ||
          settings.xAxisLabel.type === 'datetime') && (
          <div className="mb-2">
            <Input
              type="dropdown"
              label="Group Time:"
              value={settings.groupDataByTime}
              options={['', 'hourly', 'daily', 'weekly', 'monthly', 'quarterly', 'yearly']}
              onChange={value => {
                updateSettings({ groupDataByTime: value as string });
              }}
            />
          </div>
        )}
      </div>
    </div>
  );
};
